/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.phansson.dummyproxy;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;

/**
 * Executed for each TCP client which connects
 */
public class DummyProxyClientThread implements Runnable {

    private static final String CRLF = "\r\n";
    private final Socket clientSocket;
    private final int threadNo;

    public DummyProxyClientThread(Socket clientSocket, int threadNo) {
        this.clientSocket = clientSocket;
        this.threadNo = threadNo;
    }

    @Override
    public void run() {
        try (PrintWriter out = new PrintWriter(clientSocket.getOutputStream(), true);
                BufferedReader in = new BufferedReader(
                        new InputStreamReader(clientSocket.getInputStream()))
                ) {
            String inputLine;
            while ((inputLine = in.readLine()) != null) { // until client disconnects by himself
                if (inputLine.startsWith("CONNECT ") || inputLine.startsWith("GET ")) {
                    String cmd = inputLine;
                    logMe("Command \"" + cmd + "\" received from " + clientSocket.getInetAddress());
                    
                    // read rest of message from client (we don't really care what it is)
                    while (!(inputLine = in.readLine()).isEmpty()) {} 
                    
                    logMe("Sending response ...");
                    // Do a crude simulation of a HTTP response, just enough
                    // to make it work.
                    out.print("HTTP/1.1 407 Proxy Authentication Required" + CRLF);
                    out.print("Content-Type: text/html;charset=utf-8" + CRLF);
                    out.print("Content-Length: 0" + CRLF);
                    out.print("Proxy-Authenticate: Negotiate" + CRLF);
                    out.print("Via: HTTP/1.1 dummyproxy" + CRLF);
                    out.print("Connection: keep-alive" + CRLF);
                    out.print(CRLF);
                    out.flush();
                } else {
                    logMe("Invalid input received: " + inputLine);
                    break;
                }
            }
            clientSocket.close();
            logMe("Client connection closed");
        } catch (IOException ex) {
            logMe("There was an I/O error");
        }
        logMe("Thread dies");
    }

    
    private void logMe(String txt) {
        System.out.println("(#" + threadNo + ") " + txt);
    }
}
